import { useLocation, useSearch } from "wouter";
import { Button } from "@/components/ui/button.tsx";
import { GalleryCard, GalleryCardSkeleton } from "@/components/gallery/GalleryCard.tsx";
import TagBadge from "@/components/gallery/TagBadge.tsx";
import { useSearchGalleryInfinite } from "../hook/useSearchGallery.ts";
import { Spinner } from "../components/Spinner.tsx";
import TagInput from "@/components/gallery/TagInput.tsx";
import { useEffect, useRef, useState } from "react";
import { useVirtualizer } from "@tanstack/react-virtual";
import { SearchIcon, TagIcon, AlertCircle, ImageIcon } from "lucide-react";

export default function Gallery() {
    const search = useSearch();
    const searchParams = new URLSearchParams(search);
    const word = searchParams.get("word") ?? undefined;
    const tags = searchParams.getAll("allow_tag") ?? undefined;
    const limit = searchParams.get("limit");
    const cursor = searchParams.get("cursor");
    const { data, error, isLoading, size, setSize } = useSearchGalleryInfinite({
        word, tags,
        limit: limit ? Number.parseInt(limit) : undefined,
        cursor: cursor ? Number.parseInt(cursor) : undefined
    });
    const parentRef = useRef<HTMLDivElement>(null);
    const virtualizer = useVirtualizer({
        count: size,
        // biome-ignore lint/style/noNonNullAssertion: could not be null
        getScrollElement: () => parentRef.current!,
        estimateSize: (index) => {
            if (!data) return 8;
            const docs = data?.[index];
            if (!docs) return 8;
            return docs.data.length * (200 + 8) + 32 + 8 + 8 * 2; // 200px for image, 8px for gap, 32px for title, 8px for padding
        },
        overscan: 1,
    });

    const virtualItems = virtualizer.getVirtualItems();
    useEffect(() => {
        const lastItems = virtualItems.slice(-1);
        if (lastItems.some(x => x.index >= size - 1)) {
            const last = lastItems[0];
            const docs = data?.[last.index];
            if (docs?.hasMore) {
                setSize(size + 1);
            }
        }
    }, [virtualItems, setSize, size, data]);

    useEffect(() => {
        virtualizer.measure();
    }, [virtualizer, data]);


    const renderContent = () => {
        if (!data) {
            return null;
        }
        const isLoadingMore = data && size > 0 && (data[size - 1] === undefined);
        const NoResult = data.reduce((acc, x) => acc + x.data.length, 0) === 0;

        if (NoResult) {
            return <div className="flex flex-col items-center justify-center p-12 text-center">
                <ImageIcon className="w-16 h-16 text-muted-foreground mb-4 opacity-50" />
                <h3 className="text-3xl font-semibold text-muted-foreground mb-2">검색 결과가 없습니다</h3>
                <p className="text-muted-foreground">다른 검색어나 태그로 시도해보세요</p>
            </div>
        }
        else {
            return <div className="w-full relative overflow-auto h-full"
                style={{ height: virtualizer.getTotalSize() }}>
                {// TODO: date based grouping
                    virtualItems.map((item) => {
                        const isLoaderRow = item.index === size - 1 && isLoadingMore;
                        if (isLoaderRow) {
                            return <div key={item.index}
                                className="w-full flex justify-center top-0 left-0 absolute p-8"
                                style={{
                                    height: `${item.size}px`,
                                    transform: `translateY(${item.start}px)`
                                }}>
                                <span className="text-muted-foreground"><Spinner />컨텐츠를 불러오는 중...</span>
                            </div>;
                        }
                        const docs = data[item.index];
                        if (!docs) return null;
                        return <div className="w-full grid gap-2 content-start top-0 left-0 absolute mb-4" key={item.index}
                            style={{
                                height: `${item.size}px`,
                                transform: `translateY(${item.start}px)`
                            }}>
                            {docs.startCursor && (
                                <div className="bg-muted/50 rounded-lg p-2">
                                    <h3 className="text-2xl font-medium flex items-center gap-2">
                                        ID <span className="text-primary">{docs.startCursor}</span>이하
                                    </h3>
                                </div>
                            )}
                            {docs?.data?.map((x) => (
                                <GalleryCard doc={x} key={x.id} />
                            ))}
                        </div>
                    })
                }
            </div>
        }
    }

    return (<div className="p-4 grid gap-2 h-dvh overflow-auto items-start content-start" ref={parentRef}>
        <Search />

        {((word ?? "").length > 0 || tags.length > 0) &&
            <div className="bg-primary rounded-full p-2 mt-3 shadow-lg flex flex-wrap items-center gap-2">
                {word && (
                    <div className="flex items-center bg-primary-foreground/20 rounded-full px-3 py-1 text-primary-foreground">
                        <SearchIcon className="w-4 h-4 mr-2" />
                        <span className="font-medium">{word}</span>
                    </div>
                )}
                {tags && tags.length > 0 && (
                    <div className="flex items-center flex-wrap gap-1">
                        <TagIcon className="w-4 h-4 text-primary-foreground ml-2" />
                        <ul className="inline-flex flex-wrap gap-1 ml-1">
                            {tags.map(x => <TagBadge tagname={x} key={x} />)}
                        </ul>
                    </div>
                )}
            </div>
        }

        {error && (
            <div className="p-6 bg-destructive/10 rounded-lg flex items-center">
                <AlertCircle className="w-6 h-6 text-destructive mr-2" />
                <div className="text-destructive font-medium">{String(error)}</div>
            </div>
        )}

        {isLoading && (
            <div className="grid gap-2">
                <GalleryCardSkeleton />
                <GalleryCardSkeleton />
                <GalleryCardSkeleton />
                <GalleryCardSkeleton />
            </div>
        )}
        {renderContent()}
    </div>
    );
}

function Search() {
    const search = useSearch();
    const [, navigate] = useLocation();
    const searchParams = new URLSearchParams(search);
    const [tags, setTags] = useState(searchParams.get("allow_tag")?.split(",") ?? []);
    const [word, setWord] = useState(searchParams.get("word") ?? "");
    return <div className="flex flex-col sm:flex-row gap-3">
        <TagInput
            className="flex-1 shadow-sm"
            input={word}
            onInputChange={setWord}
            tags={tags}
            onTagsChange={setTags}
        />
        <Button
            className="flex-none gap-2 px-4 shadow-md hover:shadow-lg transition-all"
            onClick={() => {
                const params = new URLSearchParams();
                if (tags.length > 0) {
                    for (const tag of tags) {
                        params.append("allow_tag", tag);
                    }
                }
                if (word) {
                    params.set("word", word);
                }
                navigate(`/search?${params.toString()}`);
            }}
        >
            <SearchIcon className="w-4 h-4" />
            검색
        </Button>
    </div>
}

