import { Kysely } from "kysely";
import { getKysely } from "./db/kysely.ts";
import fs from "node:fs/promises";
import path from "path";

export async function connectDB() {
	const kysely = getKysely();

	let tries = 0;
	for (; ;) {
		try {
			console.log("try to connect db");
			await kysely.selectNoFrom(eb => eb.val(1).as("dummy")).execute();
			console.log("connect success");
		} catch (err) {
			if (tries < 3) {
				tries++;
				console.error(`connection fail ${err} retry...`);
				await new Promise((resolve) => setTimeout(resolve, 1000));
				continue;
			}
			throw err;
		}
		break;
	}
	return kysely;
}

async function checkTableExists(kysely: Kysely<any>, table: string) {
	const result = await kysely.selectFrom("sqlite_master")
		.selectAll()
		.where("type", "=", "table")
		.where("name", "=", table)
		.executeTakeFirst();
	return result !== undefined;
}

export async function migrateDB() {
	const kysely = getKysely();
	// is schema_migration exists?
	const hasTable = await checkTableExists(kysely, "schema_migration");
	if (!hasTable) {
		// 2. 마이그레이션 실행 (최초 마이그레이션)
		const migration = await import("../migrations/2024-12-27.ts");
		await migration.up(kysely);
		console.log("최초 마이그레이션 완료");
		return;
	}

	// 현재 버전 확인
	const row = await kysely.selectFrom("schema_migration").selectAll().executeTakeFirst();
	const currentVersion = row?.version ?? "0001-01-01"; // 기본값 설정

	// 마이그레이션 목록 정의 (버전순 정렬 필수)
	const migrations = await readMigrations();

	// 현재 버전보다 높은 migration만 실행
	let lastestVersion = currentVersion;
	console.log(`현재 DB 버전: ${currentVersion}`);
	for (const m of migrations) {
		if (compareVersion(m.version, currentVersion) > 0) {
			console.log(`마이그레이션 실행: ${m.version}`);
			const migration = await import(m.file);
			await migration.up(kysely);
			await kysely.updateTable("schema_migration")
				.set({ version: m.version, dirty: 0 })
				.execute();
			lastestVersion = m.version;
			console.log(`마이그레이션 완료: ${m.version}`);
		}
	}
	if (lastestVersion !== currentVersion) {
		console.log(`마이그레이션 완료. ${currentVersion} -> ${lastestVersion}`);
	} else {
		console.log("마이그레이션 필요 없음");
	}
	return;
}

async function readMigrations(): Promise<{ version: string; file: string }[]> {
	const migrationsDir = path.join(import.meta.dirname, "../migrations");
	const files = (await fs.readdir(migrationsDir))
		.filter(file => file.endsWith(".ts"))
		.map(file => {
			const version = file.match(/(\d{4}-\d{2}-\d{2})/)?.[0] || "0001-01-01";
			return { version, file: `../migrations/${file}` };
		});
	return files.sort((a, b) => compareVersion(a.version, b.version));
}

// Date 기반 버전 비교 함수. 
function compareVersion(a: string, b: string): number {
	const dateA = new Date(a);
	const dateB = new Date(b);
	if (dateA < dateB) return -1;
	if (dateA > dateB) return 1;
	return 0; // 같을 경우
}